/*********************************************************************
    This file is part of Siworin. Copyright © 2023 by Ruud Harmsen.

    Siworin is a simple word indexer and local search engine. See
    https://rudhar.com/sfreview/siworin/#siw04 and
    https://rudhar.com/sfreview/siworin/toolsrc/siworin .

    Siworin is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    Siworin is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Siworin.  If not, see <https://www.gnu.org/licenses/>.
  ********************************************************************/

/* This program reads the list of words created siworin-wordsep,
   (specify filename as first command line argument) and turns them
   into hypertext links, which invoke CGI program s.cgi (with a short
   name because it has to be mentioned so often), with the search word
   prefilled.

   Siworin-genindex also divides the complete list into NumberOfIndexFiles
   alphabetic ranges (names: index-?.html), and balances the number of
   entries in them. It also generates an iframe file (index.html) to show
   all ranges on the screen.

   14 january 2023, R. Harmsen, based on an earlier very similar program
   that still used frames, instead of iframes.

   15 january 2023:
   Encode the URL for calling the search function, so & become %26, etc.,
   so it will work.
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "cgitools.h"

#define NumberOfIndexFiles 16

int main (int argc, char **argv)
{
   char wordbuf[81];
   FILE *fpn = NULL;
   FILE *fpi = NULL;
   FILE *fpo = NULL;
   int CurrentLetter = -1;
   long words_per_file;
   long words = 0;
   int numfiles = 0;

   if (argc > 1)
   {
      fpi = fopen(argv[1], "rt");
   }
   else
   {
      fprintf(stderr, "Usage: %s file-with-words\n", argv[0]);
      return 1;
   }
   if (!fpi)
   {
      fprintf(stderr, "Cannot open %s for reading\n", argv[1]);
      return 2;
   }

   fpn = fopen("index.html", "wt");
   if (!fpn)
   {
      fprintf(stderr, "Cannot open index.htm for writing\n");
      return 3;
   }

   fprintf(fpn, "<!doctype html><html><head><title>Site index</title>\n");
   fprintf(fpn, "<meta name=\"robots\" content=\"noindex, nofollow\">\n"
                "<meta name=viewport content=\"width=device-width, initial-scale=1\">\n"
                "<meta charset=UTF-8>\n"
                "</head>\n<body>\n");

   while (fgets(wordbuf, sizeof wordbuf, fpi) == wordbuf)
   {
      words++;
   }
   words_per_file = (long)(.5 + words / (float)NumberOfIndexFiles);
   /* Correction because new file starts at new letter */
   words_per_file -= words / 27.0 / 2.0;

   fseek(fpi, 0L, SEEK_SET);

   words = words_per_file + 1;

   while (fgets(wordbuf, sizeof wordbuf, fpi) == wordbuf)
   {
      char *p;
      int c = wordbuf[0];

      if ((p = strrchr(wordbuf, '\n')) != NULL)
         *p ='\0';

      if (words > words_per_file && numfiles < NumberOfIndexFiles)
      {
         /* Detect first letter change, ignore accented letters */
         if (CurrentLetter == -1 ||
            (c >= '0' && c <= 0xfe && c != CurrentLetter))
         {
            char outname[20];

            numfiles++;
            if (fpo)
            {
               fprintf(fpo, "</body></html>\n");
               fclose(fpo);
            }
            if (!(c >= '0' && c <= 0xfe))
               c = '0';

            words -= words_per_file;

            sprintf(outname, "index-%c.html", c);
            fpo = NULL;
            fpo = fopen(outname, "wt");
            if (!fpo)
            {
               fprintf(stderr, "Cannot open %s for writing\n", outname);
               continue;
            }

            fprintf(fpo,
               "<!doctype html><html><head><title>Index %c</title>\n", toupper(c));
            fprintf(fpo,
               "<meta name=\"robots\" content=\"noindex, nofollow\">\n"
               "<meta name=viewport content=\"width=device-width, initial-scale=1\">\n"
               "<meta charset=UTF-8>\n"
               "<link rel=stylesheet href=\"../css/index.css\">\n"
               "<link rel=stylesheet href=\"../cgi-bin/colschem.cgi\">\n"
               "</head>\n<body>\n");
            fprintf(fpn,
               "\t<iframe src=%s></iframe>\n", outname);
         }
      }
      else
      {
         words++;
      }

      {
         char *ExpBuf;
      fprintf(fpo,
         "<a target=s href=\"../cgi-bin/s.cgi?q=%s\">%s</a><br>\n",
         ExpBuf = CgiToolsExpandit(wordbuf), wordbuf);
         free(ExpBuf);
      }
      if (c >= '0' && c <= 0xfe)
         CurrentLetter = c;
   }
   if (fpo)
      fclose(fpo);
   if (fpi)
      fclose(fpi);

   fprintf(fpn, "</body>\n</html>\n");
   if (fpn)
      fclose(fpn);

   return 0;
}

